/*
 *  m_operlist.c: List all (global) opers on the network or on a servermask
 *                provided.
 *
 *  Copyright 2003 by W. Campbell
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are
 *  met:
 *
 *  1.Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer. 
 *  2.Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution. 
 *  3.The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 *  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 *  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 *  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 *  $Id$
 *
 */

/* List of ircd includes from ../include/ */
#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "common.h"     /* FALSE bleah */
#include "ircd.h"
#include "irc_string.h"
#include "numeric.h"
#include "fdlist.h"
#include "s_bsd.h"
#include "s_conf.h"
#include "s_log.h"
#include "s_serv.h"
#include "send.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"
#include "s_misc.h"

static void mo_operlist(struct Client *client_p, struct Client *source_p,
                     int parc, char *parv[]);

struct Message operlist_msgtab = {
  "OPERLIST", 0, 0, 0, 0, MFLG_SLOW, 0,
#ifdef HAVE_ENCAP_HANDLERS
  {m_unregistered, m_not_oper, m_ignore, m_ignore, mo_operlist}
#else
  {m_unregistered, m_not_oper, m_ignore, mo_operlist}
#endif
};

#ifndef STATIC_MODULES
void
_modinit(void)
{
  mod_add_cmd(&operlist_msgtab);
}

void
_moddeinit(void)
{
  mod_del_cmd(&operlist_msgtab);
}

const char *_version = "$Revision$";
#endif

/*
** mo_operlist
** Syntax:  OPERLIST [servermask]
** Default:  all servers
*/
static void mo_operlist(struct Client *client_p, struct Client *source_p,
                        int parc, char *parv[])
{		 
  int count = 0;
  struct Client *target_p;
  dlink_node *dl;
  char *smatch = NULL;

  if (parc > 1)
  {
    smatch = parv[1];
  }

#ifdef CONFIG_RATBOX_LEVEL_1
  DLINK_FOREACH(dl, global_client_list.head)
  {
    target_p = dl->data;
#elif defined(CONFIG_H_LEVEL_7)
  for (target_p = GlobalClientList; target_p; target_p = target_p->next)
  {
#endif
    if (IsOper(target_p))
    {
      if (smatch != NULL)
      {
        if (match(smatch, target_p->user->server) == 0)
        {
          /* Not matched, the oper is not on the requested server */
          continue;
        }
      }
      sendto_one(source_p, ":%s %d %s p :[%c] %s (%s@%s) on %s",
                 me.name, RPL_STATSDEBUG, source_p->name,
                 IsAdmin(target_p) ? 'A' : 'O',
                 target_p->name, target_p->username, target_p->host,
                 target_p->user->server);
      count++;
    }
  }
  sendto_one(source_p, ":%s %d %s p :%d OPER(s)", me.name, RPL_STATSDEBUG,
             source_p->name, count);
}

